<?php

namespace SweepAndGo\App\Hooks;

class SngPageHook {

    private string $post_type = 'sng_page';

    public function handle() {
        add_action( 'init', [ $this, 'register_sng_page' ] );
        add_filter( 'page_row_actions', [ $this, 'ab_testing_action' ], 10, 2 );
        add_filter( 'page_row_actions', [ $this, 'ab_testing_finalize_action' ], 20, 2 );
        add_action( 'sng_spinner_element', [ $this, 'spinner_element' ], 100, 0 );
        add_action( 'add_meta_boxes_sng_page', [ $this, 'form_content_meta_box' ] );
        add_action( 'save_post_sng_page', [ $this, 'handle_save_sng_post' ], 10, 2 );
    }

    public function ab_testing_action( $actions, $post ) {
        if ( $post->post_type == 'sng_page'
            && get_post_meta( $post->ID, '_wp_page_template', true ) == 'sng-client-onboarding-template.php'
            && !get_post_meta( $post->ID, '_sng_ab_testing', true ) ) {
            $url = admin_url( "admin.php?page=sweepandgo-ab-testing&post=$post->ID" );
            $actions[ 'ab_testing' ] = "<a href='$url'>AB Testing</a>";
        }

        return $actions;
    }

    public function ab_testing_finalize_action( $actions, $post ) {
        if ( $post->post_type == 'sng_page'
            && get_post_meta( $post->ID, '_wp_page_template', true ) == 'sng-client-onboarding-template.php' ) {
            $ab_testing = get_post_meta( $post->ID, '_sng_ab_testing', true );
            if ( $ab_testing ) {
                $url = admin_url( "admin.php?page=sweepandgo-ab-testing-finalize&post=$post->ID" );
                $actions[ 'ab_testing_finalize' ] = "<a href='$url'>Use $ab_testing</a>";
            }
        }

        return $actions;
    }


    /**
     * Apply the html for plugin loader
     * @return void
     */
    public function spinner_element() {
        $action = 'wp_footer';

        if ( did_action( 'sng_spinner_element' ) > 1 ) {
            return;
        }

        // Insert in admin footer
        if ( is_admin() ) {
            $action = 'admin_footer';
        }

        add_action( $action, function () {
            $sng_settings = get_option( 'sng_settings' );
            $spinner_image = $sng_settings[ 'spinner_image' ] ?? null;
            printf( '
			<div class="sng-spinner-overlay">
				<div class="sng-spinner">
					<img class="sng-spinner-img" alt="spinner" src="%s"/>
				</div>
				<div class="sng-spinner-shadow"></div>
			</div>', ( $spinner_image ?? SNG_PLUGIN_URL_PATH . 'assets/images/poo_loader.png' ) );
        }, 100 );
    }

    /**
     * Register sng_page
     *
     * @return void
     * @since 1.0.0
     *
     */
    public function register_sng_page(): void {
        $labels = [
            'name' => 'Sweep&Go Pages',
            'singular_name' => 'Sweep&Go Page',
            'menu_name' => 'Sweep&Go Pages'
        ];

        $args = [
            'labels' => $labels,
            'exclude_from_search' => true,
            'publicly_queryable' => true,
            'show_in_nav_menus' => false,
            'show_in_admin_bar' => false,
            'show_in_menu' => false,
            'public' => true,
            'has_archive' => true,
            'description' => 'These are the pages auto generated by Sweep&Go plugin.',
            'menu_position' => null,
            'show_in_rest' => false,
            'map_meta_cap' => true,
            'capability_type' => 'post',
            'capabilities' => [
                'create_posts' => false,
            ],
            'supports' => [
                'title',
                'editor',
                'page-attributes',
                'post-formats',
                'revisions',
                'custom-fields',
                'excerpt'
            ],
            'show_ui' => true,
            'hierarchical' => true,
            'rewrite' => [
                'slug' => 'sng',
                'with_front' => false
            ],
        ];

        register_post_type( $this->post_type, $args );

        $this->add_sng_page_location_column( 'Location', 'sng_page', function ( $post_id ) {
            echo get_post_meta( $post_id, '_sng_location', true );
        }, '_sng_location', true );

        $this->add_sng_page_ab_testing_column( 'AB Testing', 'sng_page', function ( $post_id ) {
            echo get_post_meta( $post_id, '_sng_ab_testing', true );
        }, '_sng_ab_testing', true );

        if ( !get_option( 'sng_permalinks_flushed' ) ) {

            flush_rewrite_rules();
            update_option( 'sng_permalinks_flushed', 1 );
        }
    }

    public function add_sng_page_location_column( $column_title, $post_type, $cb, $order_by = false, $order_by_field_is_meta = false ) {
        // Column Header
        add_filter( 'manage_' . $post_type . '_posts_columns', function ( $columns ) use ( $column_title ) {
            $columns[ sanitize_title( $column_title ) ] = $column_title;

            return $columns;
        } );

        // Column Content
        add_action( 'manage_' . $post_type . '_posts_custom_column', function ( $column, $post_id ) use ( $column_title, $cb ) {
            if ( sanitize_title( $column_title ) === $column ) {
                $cb( $post_id );
            }
        }, 10, 2 );

        // OrderBy Set?
        if ( !empty( $order_by ) ) {

            // Column Sorting
            add_filter( 'manage_edit-' . $post_type . '_sortable_columns', function ( $columns ) use ( $column_title, $order_by ) {
                $columns[ sanitize_title( $column_title ) ] = $order_by;

                return $columns;
            } );

            // Column Ordering
            add_action( 'pre_get_posts', function ( $query ) use ( $order_by, $order_by_field_is_meta ) {
                if ( !is_admin() || !$query->is_main_query() ) {
                    return;
                }

                if ( sanitize_key( $order_by ) === $query->get( 'orderby' ) ) {
                    if ( $order_by_field_is_meta ) {
                        $query->set( 'orderby', 'meta_value' );
                        $query->set( 'meta_key', sanitize_key( $order_by ) );
                    }
                    else {
                        $query->set( 'orderby', sanitize_key( $order_by ) );
                    }
                }

                $screen = get_current_screen();
                if (
                    'edit' == $screen->base
                    && 'sng_page' == $screen->post_type
                    && !isset( $_GET[ 'orderby' ] )
                ) {
                    $query->set( 'orderby', 'date' );
                    $query->set( 'order', 'ASC' );
                }
            } );
        }
    }

    public function add_sng_page_ab_testing_column( $column_title, $post_type, $cb, $order_by = false, $order_by_field_is_meta = false ) {
        // Column Header
        add_filter( 'manage_' . $post_type . '_posts_columns', function ( $columns ) use ( $column_title ) {
            $columns[ sanitize_title( $column_title ) ] = $column_title;

            return $columns;
        } );

        // Column Content
        add_action( 'manage_' . $post_type . '_posts_custom_column', function ( $column, $post_id ) use ( $column_title, $cb ) {
            if ( sanitize_title( $column_title ) === $column ) {
                $cb( $post_id );
            }
        }, 10, 2 );
    }

    public function form_content_meta_box( $post ) {
        add_meta_box( 'sng_form_content', 'Additional Form Content', [ $this, 'form_content_html' ], 'sng_page', 'normal', 'high' );
    }

    public function form_content_html( $post ) {
        wp_nonce_field( basename( __FILE__ ), 'form_content_nonce' );
        wp_editor( get_post_meta($post->ID, 'sng_form_content', true), 'form_content', ['textarea_name' => 'sng_form_content']);
    }

    public function handle_save_sng_post( $post_id ) {
        // check for nonce to top xss
        if ( !isset( $_POST['form_content_nonce'] ) || !wp_verify_nonce( $_POST['form_content_nonce'], basename( __FILE__ ) ) ){
            return;
        }

        // check for correct user capabilities - stop internal xss from customers
        if ( ! current_user_can( 'edit_post', $post_id ) ){
            return;
        }

        // update fields
        if ( isset( $_REQUEST['sng_form_content'] ) ) {
            update_post_meta( $post_id, 'sng_form_content', wp_kses_post( $_POST['sng_form_content'] ) );
        }
    }
}
