<?php

namespace SweepAndGo\App\Http\Controllers;


use stdClass;
use SweepAndGo\App\Helpers\Location;
use SweepAndGo\App\Helpers\LocationSetting;
use SweepAndGo\App\View;
use SweepAndGo\Includes\Log;
use SweepAndGo\App\Helpers\Log as LogHelper;
use WP_Query;

class LocationsController {
    public function index() {
        $locations = ( new Location() )->all();

        View::render( 'admin/locations', $locations );
    }

    public function logs() {
        $logs = ( new LogHelper )->all();

        View::render( 'admin/logs', $logs );
    }

    public function settings() {
        View::render( 'admin/settings' );
    }

    /**
     * Oauth connection start
     * @return void
     */
    public function connect() {
        if ( !check_admin_referer( "connect-location-" . get_current_user_id() ) ||
            !current_user_can( 'manage_options' )
        ) {
            die( 'Error' );
        }

        # Update oauth state
        update_option( 'sng_oauth_state', $_REQUEST[ '_wpnonce' ] );
        $data = [
            'webhooks_url' => rest_url( 'sng/v1/sng_webhooks' ),
            'return_url' => admin_url( 'admin.php?page=sweepandgo-oauth-connect&action=sng_oauth' ),
            'state' => $_REQUEST[ '_wpnonce' ],
            'enabled_events' => [
                'lead:out_of_service_area',
                'lead:in_service_area',
                'lead:delete',
                'client:changed_status',
                'client:changed_info',
                'client:changed_address',
                'client:client_onboarding_recurring',
                'client:client_onboarding_onetime',
                'client:subscription_created',
                'client:subscription_canceled',
                'client:invoice_finalized',
                'client:client_no_assigned',
                'client:client_payment_declined',
                'notification:on_the_way_notification',
                'notification:off_schedule_notification',
                'notification:completed_job_notification',
                'notification:skipped_job_notification',
                'notification:client_not_assigned',
                'client:client_assigned',
                'client:subscription_cancel_requested',
                'client:notification_settings_changed',
                'client:reviews_automation',
                'staff:staff_clock_in',
                'staff:staff_forgot_to_clock_out',
                'organization:client_onboarding_form_changed',
                'organization:cross_sells_changed',
            ]
        ];

        wp_redirect( SNG_OPENAPI_URL . "/access_token?" . http_build_query( $data ) );
    }

    /**
     * @throws \Exception
     */
    public function delete() {
        if ( check_admin_referer( "delete-location-" . get_current_user_id() ) ||
            current_user_can( 'manage_options' )
        ) {
            $delete_location = ( new Location() )->delete( $_REQUEST[ 'location_id' ] );

            if ( $delete_location ) {
                do_action( 'sng_location_deleted', $_REQUEST );
                wp_safe_redirect( $_REQUEST[ '_wp_http_referer' ] );
                exit;
            }

            throw new \Exception( "Something wrong happened!", 500 );
        }
    }

    public function delete_settings( $request ): void {
        $sng_notice_type = 'success';
        $sng_notice_text = "<p>Location {$request['location_name']} deleted.</p>";
        $delete_settings = ( new LocationSetting() )->delete( $request[ 'location_id' ] );
        if ( !$delete_settings ) {
            $sng_notice_type = 'error';
            $sng_notice_text .= "<p>Failed to delete {$request['location_name']} settings.</p>";
        }
        $sng_pages = [
            'post_type' => [ 'sng_page' ],
            'post_status' => [ 'publish' ],
            'meta_query' => [
                [
                    'key' => '_sng_location_id',
                    'value' => $request[ 'location_id' ]
                ]
            ]
        ];

        $sng_pages_query = new WP_Query( $sng_pages );
        if ( $sng_pages_query->have_posts() ) {
            $posts = $sng_pages_query->posts;
            foreach ( $posts as $post ) {
                $delete_post = wp_delete_post( $post->ID, true );
                if ( !$delete_post ) {
                    $sng_notice_type = 'error';
                    $sng_notice_text .= "<p>Failed to delete page <strong>$post->post_title!</strong></p>";
                    Log::error( "Failed to delete $post->post_title" );
                }
                else {
                    $sng_notice_text .= "<p>Page <strong>$post->post_title</strong> deleted.</p>";
                    Log::info( "Page $post->post_title deleted." );
                }
            }
        }

        set_transient( get_current_user_id() . 'sng_location_page_notice', [
            'type' => $sng_notice_type,
            'text' => $sng_notice_text
        ], 60 );
    }

    /**
     * Filter sng_get_location_setting
     *
     * @param int    $location_id
     * @param string $key
     *
     * @return mixed|string
     * @since 1.0.0
     *
     */
    public function get_settings( int $location_id, string $key ) {
        $location_setting = ( new LocationSetting )->get_location_setting_single( $location_id, $key );

        return $location_setting;
    }

    public function get_organization_data( int $location_id, string $location_slug ) {
        $token = ( new LocationSetting() )->get_location_setting_single( $location_id, 'openapi_token' );

        $organization_data = wp_remote_request(SNG_OPENAPI_URL . '/api/v2/client_on_boarding/organization_data', [
            'method' => 'GET',
            'timeout' => 30,
            'headers' => [
                'Content-Type' => 'application/json',
                'Accept' => 'application/json',
                'Authorization' => "Bearer $token"
            ],
            'body' => [
                'organization' => $location_slug
            ]
        ]);

        if ( is_wp_error( $organization_data ) ||
            ! in_array( wp_remote_retrieve_response_code( $organization_data ), [200, 201] ) ) {
            Log::error('Error getting the organization data', json_encode($organization_data));
            error_log(json_encode($organization_data));

            return false;
        }

        return json_decode( wp_remote_retrieve_body( $organization_data ) ) ;
    }
}