<?php

namespace SweepAndGo\App\Http\Controllers;

use SweepAndGo\App\View;
use SweepAndGo\Packages\OpenApi;
use SweepAndGo\App\Helpers\Location;
use SweepAndGo\App\Helpers\LocationSetting;
use SweepAndGo\Includes\Log;

class OauthHandlerController {
	public function oauth_connect(): void {
		if (
			isset( $_REQUEST['action'] ) &&
			isset( $_REQUEST['state'] ) &&
			isset( $_REQUEST['code'] )
		) {
			if ( $_REQUEST['action'] == 'sng_oauth' ) {
				$wp_state = get_option('sng_oauth_state');

				if ( $wp_state != $_REQUEST['state'] ) {
					wp_die("Invalid request");
				}

				Log::info('Oauth connection', $_REQUEST);

				$token_response = wp_remote_request(SNG_OPENAPI_URL . "/access_token_complete?code=$_REQUEST[code]", [
					"method" => "GET",
					'timeout' => 60,
					"headers" => [
						"Content-Type" => "application/json",
						"Accept" => "application/json",
					],
				]);

				if ( is_wp_error( $token_response ) ||
				     ! in_array( wp_remote_retrieve_response_code( $token_response ), [200, 201] ) ) {
					// TODO Log error and throw exception
					Log::error( "OpenAPI Location connect error", $token_response );
				}

				$response = json_decode( wp_remote_retrieve_body( $token_response ) );
				add_action('sng_after_oauth_connection_action', [ $this, 'oauth_connect_action'], 10, 1 );
				do_action('sng_after_oauth_connection_action', $response);
				remove_action('sng_after_oauth_connection_action', [ $this, 'oauth_connect_action'], 10 );
			}
		}
		$this->handle_oauth_redirect();
	}

	private function handle_oauth_redirect() {
		do_action('sng_after_request_notice_action');
		if ( headers_sent() ) {
			View::render('admin/oauth_page');
		} else {
			wp_safe_redirect( admin_url( 'admin.php?page=sweepandgo-locations' ) );
			exit();
		}
	}

	public function oauth_connect_action( $response ): void {
		$open_api = new OpenApi( $response->token );

		$location_exists = ( new Location() )->first('slug', '=',  $response->org_slug );

		if ( $location_exists ) {
			set_transient( get_current_user_id() . 'sng_location_page_notice', [
				'type' => 'error',
				'text' => '<p>This location is already added!</p>'
			], 10 );
			$this->handle_oauth_redirect();
			exit;
		}

		$location_plan_check = $open_api->location_plan_check( sanitize_title( $response->org_slug ) );

		if ( ! $location_plan_check->valid ) {
			set_transient( get_current_user_id() . 'sng_location_page_notice', [
				'type' => 'error',
				'text' => '<p>Invalid onboarding URL! Please double-check your client onboarding URL and Open API token. Additionally, ensure that your business is subscribed to the POOfessional plan.<p>'
			], 10 );
			$this->handle_oauth_redirect();
			exit;
		}
		$name = sanitize_text_field( $location_plan_check->name );

		global $sng_location;
		$sng_location = (new Location())->create([
			'name' => $name,
			'slug' => $response->org_slug
		]);

		if ( is_wp_error( $sng_location ) ) {
			set_transient( get_current_user_id() . 'sng_location_page_notice', [
				'type' => 'error',
				'text' => "<p>{$sng_location->get_error_message()}</p>",
			], 10 );
			$this->handle_oauth_redirect();
		}

		$settings_data = [
			'openapi_token' => $response->token,
			'valid_plan' => $location_plan_check->valid,
			'sng_org_id' => $response->org_id
		];

		$this->create_default_settings( $settings_data, $sng_location );

		set_transient( get_current_user_id() . 'sng_location_page_notice', [
			'type' => 'success',
			'text' => '<p>New location is successfully added.</p>'
		], 10 );

		// Flush rewrite rules
		flush_rewrite_rules();
	}

	/**
	 * Create settings for new location with default parameters
	 * @param array $settings_data
	 * @param int $location_id
	 * @return void
	 */
	public function create_default_settings( array $settings_data, int $location_id ): void {
		$args = [
			...$settings_data
		];

		foreach ( $args as $key => $value ) {
			( new LocationSetting() )->create([
				'setting_key' => $key,
				'setting_value' => maybe_serialize( $value ),
				'location_id' => $location_id,
				'created_at' => current_time('mysql'),
				'updated_at' => current_time('mysql'),
			]);
		}
	}
}