<?php

namespace SweepAndGo\Includes;

// Update checker
require SNG_PLUGIN_DIR_PATH . '/packages/plugin-update-checker/plugin-update-checker.php';
use YahnisElsts\PluginUpdateChecker\v5\PucFactory;

// Sweep&Go
use SweepAndGo\App\Hooks\JsModuleHook;
use SweepAndGo\App\Hooks\MenuHook;
use SweepAndGo\App\Hooks\RequestsHook;
use SweepAndGo\App\Hooks\SngPageHook;
use SweepAndGo\App\Http\Controllers\AddonController;
use SweepAndGo\App\Http\Controllers\Api\RestApiController;
use SweepAndGo\App\Http\Controllers\LocationsController;
use SweepAndGo\App\Http\Controllers\LogsController;
use SweepAndGo\App\Http\Controllers\OauthHandlerController;
use SweepAndGo\App\Http\Controllers\SettingsController;
use SweepAndGo\Database\Migrator;

class Plugin {
	private RestApiController $rest_api_controller;
	private AddonController $addon_controller;
	private LocationsController $locations_controller;
	private LogsController $logs_controller;
	private SettingsController $settings_controller;
	private OauthHandlerController $oauth_handler_controller;

	public function __construct() {
		$this->register_update_checker();
		$this->register_controllers();
		$this->global_actions();
		( new JsModuleHook() )->handle();
		if ( ! $this->plugin_init_check() ) {
			return;
		}

		$this->define_admin_hooks();
		$this->register_sng_page();
		$this->register_rest_api();
		add_action('wp_initialize_site', [$this, 'sng_migration_on_new_blog']);
	}

	private function register_update_checker() {
		$sng_update_checker = PucFactory::buildUpdateChecker(
			'https://sng-plugins.s3.us-west-2.amazonaws.com/sweep-and-go/live/info.json',
			SWEEP_AND_GO_PLUGIN_FILE, //Full path to the main plugin file or functions.php.
			'sweep-and-go'
		);
	}

	public function register_controllers() {
		$this->rest_api_controller      = new RestApiController();
		$this->addon_controller         = new AddonController();
		$this->locations_controller     = new LocationsController();
		$this->logs_controller          = new LogsController();
		$this->settings_controller      = new SettingsController();
		$this->oauth_handler_controller = new OauthHandlerController();
	}

	private function global_actions() {
		add_action( 'sng_db_check', [ $this, 'sng_db_check' ], 10, 1 );
		add_action( 'sng_db_error', [ $this, 'sng_db_error' ], 10, 1 );
		add_action( 'sng_system_info', [ $this, 'sng_system_info' ], 10, 0 );
	}

	private function plugin_init_check() {
		do_action( 'sng_db_check', [
			'locations',
			'location_settings',
			'logs'
		] );

		if ( get_option( "sng_log_enable" ) ) {
			add_action( 'admin_notices', [ $this, 'sng_logging_notice' ] );
		}

		return true;
	}

	/**
	 * Create menu and add requests
	 *
	 * @return void
	 * @since 1.0.0
	 *
	 */
	private function define_admin_hooks(): void {
		// Admin Menu
		( new MenuHook( $this->locations_controller,
			$this->addon_controller,
			$this->oauth_handler_controller,
			$this->settings_controller) )->handle();

		// Admin Requests
		( new RequestsHook( $this->locations_controller, $this->logs_controller,
			$this->addon_controller, $this->settings_controller ) )->handle();
	}

	/**
	 * Register sng_page post type
	 *
	 * @return void
	 * @since 1.0.0
	 *
	 */
	private function register_sng_page() {
		( new SngPageHook() )->handle();
	}

	private function register_rest_api() {
		add_action( 'rest_api_init', [ $this->rest_api_controller, 'register_routes' ] );
		add_action( 'sng_register_webhook_action', [ $this->rest_api_controller, 'register_webhook_actions' ] );
	}

	public function sng_migration_on_new_blog( $site ) {
		if ( is_plugin_active_for_network( 'sweep-and-go/sweep-and-go.php' ) ) {
			switch_to_blog($site->blog_id);
			Migrator::migrate();
			restore_current_blog();
		}
	}

	public function sng_db_check( array $required_tables ): void {
		global $wpdb;
		$plugin_prefix   = SNG_TABLE_PREFIX;
		$required_tables = array_map( function ( $value ) use ( $plugin_prefix, $wpdb ) {
			return "{$wpdb->prefix}{$plugin_prefix}{$value}";
		}, $required_tables );
		$tables          = $wpdb->get_results( "SHOW TABLES LIKE '{$wpdb->prefix}{$plugin_prefix}%'", OBJECT_K );
		$missing_tables  = array_diff( $required_tables, array_keys( $tables ) );
		if ( count( $missing_tables ) > 0 ) {
			do_action( 'sng_db_error', $missing_tables );
		}
	}

	public function sng_db_error( array $missing_tables ) {
		add_action( 'admin_notices', function () use ( $missing_tables ) {
			printf( '<div class="notice notice-error">
			<p>Our plugin couldn\'t create a new database table that is required for this plugin to work.<br/>
			Tables:<br/>
			<b>%1$s</b><br/>
			Please try to reactivate the plugin or contact us on 
			<a href="mailto:hello@sweepandgo.com">hello@sweepandgo.com</a> for further help.
			</p>
			</div>', ...$missing_tables );
		} );
	}

	public function sng_logging_notice() {
		$message = __( 'Warning! The Sweep&Go plugin logs are enabled. Since logs may contain sensitive information and can degrade the plugin performance, please ensure that you only leave it enabled for as long as it is needed for troubleshooting.', SNG_TEXT_DOMAIN );

		printf( '<div class="notice notice-warning""><p>%1$s</p></div>', esc_html( $message ) );
	}

	public function sng_system_info() {
		global $wp_version, $wp_db_version;
        $wp_cron = ( defined('DISABLE_WP_CRON') && DISABLE_WP_CRON ) ? 'false': 'true';
		$plugins = "";
		foreach ( get_plugins() as $plugin ) {
			$plugins .= "$plugin[Name] - v$plugin[Version]\n";
		}
		$plugin_version = SNG_VERSION;
		printf("Site URL: %s\nPlugin version: %s\nPHP version: %s\nWordPress version: %s\nWordPress DB version: %s
WordPress Cron Enabled: %s\nTheme: %s\n\nPlugins:\n%s",
			site_url(), $plugin_version, phpversion(), $wp_version, $wp_db_version, $wp_cron, wp_get_theme()['Name'], $plugins);
	}
}