<?php

namespace SweepAndGo\Packages;

use SweepAndGo\Includes\Log;

class OpenApi {
	private string $token;

	public function __construct( string $token ) {
		if ( $this->check_connection( $token ) )
			$this->token = $token;
		else {
			wp_die('Invalid token');
		}
	}

	private function check_connection( string $token ) {
		return true;
	}

	/**
	 * Check is token valid
	 *
	 * @param string $token
	 *
	 * @return mixed|null
	 */
	public function check_token( string $token ) {
		$response = $this->send_request( 'v2/check_token', 'GET', [
			'token' => $token ] );

		return $response;
	}

	/**
	 * List of all active clients
	 *
	 * @param int $page
	 *
	 * @since 1.0.0
	 *
	 * @return mixed|null
	 */
	public function get_active_clients( int $page = 1 ) {
		$response = $this->send_request( 'v1/clients/active', 'GET', [
			'page' => $page
		] );

		return $response;
	}

	/**
	 * List of all active clients without active subscription
	 *
	 * @param int $page
	 *
	 * @since 1.0.0
	 *
	 * @return mixed|null
	 */
	public function get_active_clients_with_no_subscription( int $page = 1 ) {
		$response = $this->send_request( 'v1/clients/active_no_subscription', 'GET', [
			'page' => $page
		] );

		return $response;
	}

	/**
	 * List of all inactive clients
	 *
	 * @param int $page
	 *
	 * @since 1.0.0
	 *
	 * @return mixed|null
	 */
	public function get_inactive_clients( int $page = 1 ) {
		$response = $this->send_request( 'v1/clients/inactive', 'GET', [
			'page' => $page
		] );

		return $response;
	}

	/**
	 * Check is location on the highest(Poofessional) plan
	 *
	 * @param string $slug
	 *
	 * @since 1.0.0
	 *
	 * @return object|null
	 */
	public function location_plan_check( string $slug ) {
		$response = $this->send_request( 'v2/check_premium_organization', 'GET', [
			'slug' => $slug
		] );

		return $response;
	}

	/**
	 * Register new residential client
	 *
	 * @param array $data
	 *
	 * @since 1.0.0
	 *
	 * @return object|null
	 */
	public function sign_up_client( array $data ) {
		$response = $this->send_request( 'v1/residential/onboarding', 'PUT', json_encode( $data ) );

		return $response;
	}

	/**
	 * Get thank you pages content
	 *
	 * @param string $slug
	 *
	 * @since 1.0.0
	 *
	 * @return object|null
	 */
	public function get_sng_thank_you_pages( string $slug ) {
		$response = $this->send_request( 'v2/client_on_boarding/thank_you_pages', 'GET', [
			'slug' => $slug
		] );

		return $response;
	}

	/**
	 * Get onboarding form settings from Sweep&Go OpenAPI
	 *
	 * @since 1.0.0
	 *
	 * @param string $slug
	 *
	 * @return object
	 */
	public function get_sng_form( string $slug ) {
		$response = $this->send_request( 'v2/client_on_boarding/service_registration_form', 'GET', [
			'organization' => $slug
		]);

		return $response;
	}

	/**
	 * Get onboarding form settings from Sweep&Go OpenAPI
	 * @param string $slug
	 * @return object
	 */
	public function get_sng_out_of_area_form( string $slug ) {
		$response = $this->send_request( 'v2/client_on_boarding/out_of_service_form', 'GET', [
			'organization' => $slug
		]);

		return $response;
	}

	/**
	 * Get location payment processor from Sweep&Go OpenAPI
	 *
	 * @since 1.0.0
	 *
	 * @param string $slug
	 *
	 * @return string
	 */
	public function get_location_payment_processor( string $slug ): string {
		$response = $this->send_request( 'v2/client_on_boarding/organization_data', 'GET', [
			'organization' => $slug
		]);

		if ( !is_null( $response->organization->card_connect_site ) ) {
			return 'card_connect';
		}

		return 'stripe';

	}

	/**
	 * Get all Leads
	 * This endpoint returns all leads with pagination.
	 *
	 * @param int $page
	 *
	 * @return mixed|null
	 */
	public function get_all_leads( int $page = 1 ): mixed {
		$response = $this->send_request( 'v1/leads/list', 'GET', [
			'page' => $page
		] );

		return $response;
	}

	/**
	 * Get Out of area leads
	 * This endpoint returns all leads who are Out of area with pagination.
	 *
	 * @param int $page
	 *
	 * @return mixed|null
	 */
	public function get_out_of_area_leads( int $page = 1 ): mixed {
		$response = $this->send_request( 'v1/leads/out_of_service', 'GET', [
			'page' => $page
		] );

		return $response;
	}

	/**
	 * Dispatch Board
	 *
	 * This endpoint returns jobs list for a selected date - Dispatch Board for selected date.
	 *
	 * If returned id is bigger than zero that means that the job is already dispatched. If it’s zero, it means that job is not dispatched.
	 *
	 * Job status can be: pending(1), completed(2), skipped(3), missed(4), started(5) and dispatched(6)
	 *
	 * Job type can be: custom, initial, one_time, reclean and recurring
	 *
	 * @param string $date
	 *
	 * @return mixed|null
	 */
	public function get_dispatch_board_for_date( string $date ) {
		$response = $this->send_request( 'v1/dispatch_board/jobs_for_date', 'GET', [
			'date' => $date
		] );

		return $response;
	}

	private function send_request( string $endpoint, string $method, array|string $data = [] ) {
		$request = wp_remote_request( SNG_OPENAPI_URL . "/api/$endpoint", [
			'method'    => $method,
			'body'      => $data,
			'timeout'   => 30,
			'headers'   => [
				'Content-Type' => 'application/json',
				'Accept' => 'application/json',
				'Authorization' => "Bearer $this->token",
			]
		]);
		if ( is_wp_error( $request ) ||
		     ! in_array( wp_remote_retrieve_response_code( $request ), [200, 201] ) ) {
			// TODO Log error and throw exception
			Log::error( "Error submitting request to OpenAPI", json_encode($request) );

			return [
				'success' => false,
				'data' => wp_remote_retrieve_body( $request )
			];
		}

		return json_decode( wp_remote_retrieve_body( $request ) );
	}

}