<?php

require_once( ABSPATH . 'wp-admin/includes/class-wp-list-table.php' );

class LogsTable extends WP_List_Table {

	private array $data;

	private string $logs_table;

	private wpdb $wpdb;

	public function __construct( $args = [] ) {
		parent::__construct( $args );
		global $wpdb;
		$this->wpdb = $wpdb;
		$this->logs_table = $this->wpdb->base_prefix . SNG_TABLE_PREFIX . 'logs';
	}

	public function get_columns() {
		$columns = [
			'cb' => '<input type="checkbox" />',
			'id' => 'ID',
			'data' => 'DATA',
			'message' => 'MESSAGE',
			'type' => 'TYPE',
			'created_at' => 'DATE SUBMITTED'
		];

		return $columns;
	}

	public function prepare_items() {
		/**
		 * Verify delete/retry actions nonce
		 */
		if ( isset( $_REQUEST['action'] ) ) {
			if ( $this->request_verify( $_REQUEST ) ) {
				$this->{$_REQUEST['action']}();
			}
		}

		/**
		 * Verify filter nonce
		 */
		if ( isset( $_REQUEST['filters'] ) ) {
            $_REQUEST['filters'] = explode( ',', $_REQUEST['filters'] );
			$this->data = $this->get_table_data( $_REQUEST['filters'] );
		}
		else
			$this->data = $this->get_table_data();


		$columns = $this->get_columns();
		$hidden = [];
		$sortable = $this->get_sortable_columns();
		$this->_column_headers = [$columns, $hidden, $sortable];
		usort( $this->data, [&$this, 'usort_reorder'] );

		/* Pagination */
		$per_page = $this->get_items_per_page( 'logs_per_page', 5 );
		$current_page = $this->get_pagenum();
		$total_items = count( $this->data );

		$this->data = array_slice( $this->data, ( ( $current_page - 1 ) * $per_page ), $per_page );
		$this->set_pagination_args( [
			'total_items' => $total_items, // total number of items
			'per_page' => $per_page, // items to show on a page
			'total_pages' => ceil( $total_items / $per_page ) // use ceil to round up
		] );

		$this->items = $this->data;
	}

	public function column_default( $item, $column_name ) {
		switch ( $column_name ) {
			case 'id':
			case 'data':
			case 'name':
			case 'type':
			case 'created_at':
			default:
				return $item[$column_name];
		}
	}

	function column_cb( $item ) {
		return sprintf(
			'<input type="checkbox" name="sng_log_id[]" value="%s" />',
			$item['id']
		);
	}

	protected function get_sortable_columns() {
		$sortable_columns = [
			'id' => ['id', true],
			'name' => ['name', false],
			'type' => ['type', false],
			'created_at' => ['created_at', false]
		];
		return $sortable_columns;
	}

	public function usort_reorder( $a, $b ) {
		// If no sort, default to date created
		$orderby = ( !empty( $_GET['orderby'] ) ) ? $_GET['orderby'] : 'created_at';

		// If no order, default to dec
		$order = ( !empty( $_GET['order'] ) ) ? $_GET['order'] : 'desc';

		// Determine sort order
		$result = strcmp( $a[$orderby], $b[$orderby] );

		// Send final sort direction to usort
		return ( $order === 'asc' ) ? $result : -$result;
	}

	private function get_table_data( array $filters = [] ) {
		if ( empty( $filters ) ) {
			$query = "SELECT * FROM $this->logs_table";
		} else {
			$filters_placeholder = implode( ', ', array_fill( 0, count( $filters ), '%s' ) );
			$query = "SELECT * FROM $this->logs_table WHERE type IN ({$filters_placeholder})";
			$query = $this->wpdb->prepare( $query, $filters );
		}
		return $this->wpdb->get_results( $query, ARRAY_A );
	}

	public function column_id( array $item ): string {
		$nonce = wp_create_nonce( 'bulk-' . $this->_args['plural'] );
		$actions = [
			'delete' => sprintf( '<a href="?page=%s&action=%s&sng_log_id=%s&_wpnonce=%s">' . 'Delete' . '</a>', $_REQUEST['page'], 'sng_delete_log', $item['id'], $nonce )
		];
		return sprintf( '%1$s %2$s', $item['id'], $this->row_actions( $actions ) );
	}

	public function column_type( array $item ): string {
		$type = $this->sng_human_readable_type( $item['type'] );
		return sprintf( '<span style="color: %s;">%s</span>', esc_attr( $type['color'] ), esc_attr( $type['label'] ) );
	}

	public function column_name( array $item ): string {
		return sprintf( '%s',( $item['name'] ) ?? 'deleted' );
	}

	public function column_data( array $item ): void {
		echo sprintf( '<a class="sng-show-log-data" href="#show-%1$d" data-log-id="%1$d">Show Data</a>', $item['id'] );
		echo sprintf( '<div class="data" style="display:none;" data-log-id="%d">', $item['id'] );
        $data = maybe_unserialize( $item["data"] );
		if ( is_array( $data ) && ! empty( $data ) ) {
			foreach ( $data as $key => $item ) {
                if ( is_array( $item ) || is_object( $item ) ) {
	                "<b>$key:</b>" . json_encode( $item ) . "<br>";
                } else {
	                echo "<b>$key:</b> $item<br>";
                }
			}
        } else if ( is_string( $data ) && $data !== '' ) {
            echo $data;
        } else {
            echo "/";
        }
		echo "</div>";
	}

	public function get_bulk_actions(): array {
		$actions = [
			'sng_delete_log' => 'Delete',
		];
		return $actions;
	}

	/**
	 * Add filters form
	 * @param $which
	 * @return void
	 */
	public function extra_tablenav( $which ): void {
		if ( $which == 'top' ) {
            if ( !empty( $_REQUEST['orderby'] ) ) {
                echo '<input type="hidden" name="orderby" value="' . esc_attr( $_REQUEST['orderby'] ) . '" />';
            }
            if ( !empty( $_REQUEST['order'] ) ) {
                echo '<input type="hidden" name="order" value="' . esc_attr( $_REQUEST['order'] ) . '" />';
            }
            ?>
            <div class="alignleft actions filter-actions">
                <select name="filters[]" id="sng_filters" class="sng-select-control" multiple
                        data-placeholder="FILTERS">
                    <option></option>
                    <option
                        <?php echo ( !empty( $_REQUEST['filters'] ) && in_array( 'error', $_REQUEST['filters'] ) ) ? 'selected' : null; ?>
                            value="error">Error
                    </option>
                    <option
                        <?php echo ( !empty( $_REQUEST['filters'] ) && in_array( 'info', $_REQUEST['filters'] ) ) ? 'selected' : null; ?>
                            value="info">Info
                    </option>
                </select>
                <input type="button" id="dofilteraction" class="button action" value="Apply">
            </div>
            <?php
		}
	}

	/**
	 * Delete single/multiple logs
	 * @return void
	 */
	public function sng_delete_log(): void {
		if ( is_array( $_REQUEST['sng_log_id'] ) ) {
			foreach ( $_REQUEST['sng_log_id'] as $log_id ) {
				$this->wpdb->delete( $this->logs_table, ['id' => $log_id], '%d' );
			}
		} else {
			$this->wpdb->delete( $this->logs_table, ['id' => $_REQUEST['sng_log_id']] );
		}
	}

	/**
	 * Verify request nonce
	 * @param $request
	 * @return bool
	 */
	private function request_verify( $request ): bool {
		return wp_verify_nonce( $request['_wpnonce'], 'bulk-' . $this->_args['plural'] ) || !current_user_can( 'manage_options' );
	}

	private function sng_human_readable_type( $key ) {
		$types = [
			'error' => [
				'label' => 'ERROR',
				'color' => 'red'
			],
			'info' => [
				'label' => 'INFO',
				'color' => 'blue'
			],
		];

		return $types[$key];
	}
}